@@ -3,6 +3,14 @@ package Benchmark::Perl::Formance::Plugin::DPath;
 use strict;
 use warnings;
 
+our $VERSION = "0.001";
+
+#############################################################
+#                                                           #
+# Benchmark Code ahead - Don't touch without strong reason! #
+#                                                           #
+#############################################################
+
 our $goal;
 our $count;
 
@@ -418,7 +426,8 @@ sub run_dpath
 {
         my ($options) = @_;
 
-        eval "use Data::DPath 'dpath'";
+        eval "use Data::DPath 'dpath'"; ## no critic
+
         if ($@) {
                 print STDERR "# ".$@ if $options->{verbose} > 2;
                 return { failed => "use Data::DPath failed" };
