@@ -6,57 +6,72 @@ use 5.008;
 use strict;
 use warnings;
 
+our $VERSION = "0.001";
+
+#############################################################
+#                                                           #
+# Benchmark Code ahead - Don't touch without strong reason! #
+#                                                           #
+#############################################################
+
 our $goal;
 our $count;
 our $threadcount;
 our $val;
-our $expect;
+our $expect2;
 
 use threads;
-use threads::shared;
 
-my $result : shared;
+my $result;
 
 use Benchmark ':hireswallclock';
 use Data::Dumper;
 
-sub thread_storm
+sub run_thread_storm_noshared
 {
         my ($options) = @_;
 
-        $result = 0;
         my @t;
         foreach (1..$threadcount) {
                 push @t, async {
-                        #print STDERR "." if $options->{verbose} >= 3;
-                        $result += $val;
+                        # print STDERR ".";
+                        1; # no-op
                 }
         }
         foreach (@t) {
                 $_->join;
         }
 
-        #print STDERR "#  == $result\n" if $options->{verbose} >= 3;
-        return $result;
+        return scalar @t;
 }
 
-sub main
+sub threadstorm
 {
         my ($options) = @_;
 
         $goal        = $options->{fastmode} ? 3 : 25;
         $threadcount = $options->{fastmode} ? 5 : ($options->{D}{Threads_threadcount} || 100);
         $val         = 25;
-        $expect      = $threadcount * $val;
+        $expect2     = $threadcount;
 
-        my $ret;
-        my $t = timeit($goal, sub { $ret = thread_storm($options) });
+        my $ret2;
+        my $t2 = timeit($goal, sub { $ret2 = run_thread_storm_noshared($options) });
 
         return {
-                Benchmark   => $t,
+                Benchmark   => $t2,
                 threadcount => $threadcount,
-                result      => $ret,
-                expect      => $expect,
+                result      => $ret2,
+                expect      => $expect2,
+                useforks => ($options->{useforks} || 0),
+               };
+}
+
+sub main
+{
+        my ($options) = @_;
+
+        return {
+                threadstorm => threadstorm($options),
                };
 }
 
